#include <GL/glut.h>
#include <stdlib.h>

#include "GameObj.h"
#include "Bullet.h"
#include "Ship.h"
#include "TriShip.h"
#include "Ship3D.h"
#include "Ast.h"
#include "AstFract.h"
#include "Ast3DFract.h"
#include "Exp.h"
#include "GameSession.h"
#include "GameIntro.h"
#include "extern.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

void GameSession::Key(unsigned char key)
{
	switch(key){
		case ' ' : Shoot(); break;
		case 'p' : Pause(); break;
		}
}

void  GameSession::SKey(int key)
{
  int km = glutGetModifiers();
	switch (key) {
  case GLUT_KEY_LEFT: 
	  if(km & GLUT_ACTIVE_ALT) S->StartStrafeLeft();
			else S->StartLeft();
	  break;
	case GLUT_KEY_RIGHT: 
		if(km & GLUT_ACTIVE_ALT) S->StartStrafeRight();
			else  S->StartRight();
	  break;
  case GLUT_KEY_UP: 
	  S->ThrustOn();
	  break;
	}
}

void  GameSession::SKeyUp(int key)
{
 switch (key) {
  case GLUT_KEY_LEFT: 
	  S->StopStrafeLeft();
		S->StopLeft();
	  break;
  case GLUT_KEY_RIGHT: 
	  S->StopStrafeRight();
		S->StopRight();
	  break;
  case GLUT_KEY_UP: 
	  S->ThrustOff();
	  break;
	}
}
void  GameSession::Reshape(int w,int h)
{
  ScreenW=w;
  ScreenH=h;
	float wr=float(w)/h;
  float hr=float(h)/w;

	if(w>h) hr=1;
	else wr=1;
  int VL = -SpaceSize*wr;
  int VR =  SpaceSize*wr;
  int VB = -SpaceSize*hr;
  int VT =  SpaceSize*hr;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(VL,VR,VB,VT,-100,100);
  glMatrixMode(GL_MODELVIEW);
}
/** The main drawing function.
It draw the current ship, all the active bullets,
asteroids and  generic objects
*/
void GameSession::Draw()
{
	if(S->active)
		S->Draw();

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		(*lib)->Draw();
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		(*lia)->Draw();
	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		(*li)->Draw();	
}

void GameSession::Update(int dt)
{
	if(PauseFlag) return;
	if(S->active){
		S->Update(dt);
		Clip(S->p);
	} else {
		deadpause-=dt;
		if(deadpause<0){
			Game=&GI;
			GI.Init();
			}
	}

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		{
		 (*lib)->Update(dt);
			Clip((*lib)->p);
			BulletAstCollision(*lib);
			if((*lib)->active==false)
			{
				GameObj *todel=*lib;
				lib=ActiveBullet.erase(lib);
				S->TerminateBullet();
				delete todel;
			}
			if(lib==ActiveBullet.end()) break;
		}
	
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		{
		 (*lia)->Update(dt);
			Clip((*lia)->p);
			AstShipCollision(*lia);
			if((*lia)->active==false)
		 {
			 GameObj *todel=*lia;
			 lia=ActiveAst.erase(lia);
			 delete todel;
		 }
		 if(lia==ActiveAst.end()) break;
		}

	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		{
		 (*li)->Update(dt);
			Clip((*li)->p);
		 if((*li)->active==false)
		 {
			 GameObj *todel=*li;
			 li=ActiveObj.erase(li);
			 delete todel;
		 }
		 if(li==ActiveObj.end()) break;
		}

}

void GameSession::Shoot()
{
 S->Shoot(ActiveBullet);
}

void GameSession::Init()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	float lp[4]={1,1,3,0};
	glLightfv(GL_LIGHT0,GL_POSITION,lp);
  glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	float lc[4]={1,.7,.5,1};
	for(int i=0;i<3;i++)
		{
			glLightfv(GL_LIGHT1+i,GL_DIFFUSE,lc);
			glLightf(GL_LIGHT1+i,GL_QUADRATIC_ATTENUATION,.001f);
		}	
	glEnable(GL_CULL_FACE);
	// Remove Comments if some face disappears...
	//glDisable(GL_CULL_FACE);
	//glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

	MediumAst.clear();
	SmallAst.clear();
	ActiveAst.clear();
	ActiveBullet.clear();
	ActiveObj.clear();
	
	Exp::LightOffAll();
	S=new Ship3D("newastro.rawc");
	Ast *a;
	for(i=0;i<AstNum;i++)
		{
			a=new Ast3DFract("tetra.raw",Ast::LargeSize);
			a->p.x()=drand48()*SpaceSize*2 - SpaceSize;	
			a->p.y()=drand48()*SpaceSize*2 - SpaceSize;
			a->p.z()=0;
			a->v.x()=drand48()*40 - 20;	
			a->v.y()=drand48()*40 - 20;
			a->v.z()=0;
			a->av=20+drand48()*70;
			a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
			ActiveAst.push_back(a);
		}
	for(i=0;i<AstNum*2;i++)
	{
		a=new Ast3DFract("tetra.raw",Ast::MediumSize);
		MediumAst.push_back(a);
	}
	for(i=0;i<AstNum*4;i++)
	{
		a=new Ast3DFract("tetra.raw",Ast::SmallSize);
		SmallAst.push_back(a);
	}

}
void GameSession::AstShipCollision(Ast *a)
{
	if(S->active && a->Collide(S)) // Asteroid - Ship Collision
			{
				S->active=false;
				deadpause=2000;
				Exp *e =new Exp();
				e->p=(a->p+S->p)/2;
				ActiveObj.push_back(e);	
			}
}

void GameSession::BulletAstCollision(Bullet *b)
{
	if(!b->active) return;
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		if((*lia)->active && (*lia)->Collide(b))
			{
				(*lia)->active=false;
				b->active=false;
				Exp *e =new Exp();
				e->p=(*lia)->p;
				e->LightOn();
				ActiveObj.push_back(e);	
				if((*lia)->size>=Ast::MediumSize)
				{
					Ast *a;
					Point3f nv;
					if((*lia)->size==Ast::LargeSize)  { a=MediumAst.front();	MediumAst.pop_front();	}
					if((*lia)->size==Ast::MediumSize) { a=SmallAst.front(); 	SmallAst.pop_front();		}
					nv.x()=drand48()*40 - 20;	
					nv.y()=drand48()*40 - 20;
					nv.z()=0;
					a->av=20+drand48()*70;
					a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					a->p=(*lia)->p;
					a->v=(*lia)->v+nv;
					ActiveAst.push_front(a);
					if((*lia)->size==Ast::LargeSize)  { a=MediumAst.front();	MediumAst.pop_front();	}
					if((*lia)->size==Ast::MediumSize) { a=SmallAst.front(); 	SmallAst.pop_front();		}
					a->av=20+drand48()*70;
					a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					a->p=(*lia)->p;
					a->v=(*lia)->v-nv;
					ActiveAst.push_front(a);
				}
				return;
			}
}